<?php

declare(strict_types=1);

namespace Drupal\css_custom_properties\Form;

/**
 * @file
 * Contains Drupal\css_custom_properties\Form\CustomPropsSettingsForm.
 */

use Drupal\Core\Config\Config;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Settings form for CSS Custom Properties.
 */
class CustomPropsSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'css_custom_properties_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return [
      'css_custom_properties.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildForm($form, $form_state);
    $config = $this->getConfig();
    $properties = $config->get('properties') ?? [];

    $form['custom_property_prefix'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Custom Property Prefix'),
      '#description' => $this->t('Add a prefix to the properties created.'),
      '#default_value' => $config->get('custom_property_prefix') ?? '',
      '#required' => TRUE,
    ];

    if (!empty($properties)) {
      $form['custom_property_list'] = [
        '#type' => 'details',
        '#title' => $this->t('Custom Properties'),
        '#open' => FALSE,
      ];
    }

    foreach ($properties as $property_id => $property) {
      $widget = $this->getColorWidget(
        $property_id,
        $property['label'],
        $property['color']
      );

      $form['custom_property_list'] = array_merge(
        $form['custom_property_list'],
        $widget
      );
    }

    $form['custom_property_add_new'] = $this->getColorWidget();

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->getConfig();
    $prefix = $form_state->getValue('custom_property_prefix');
    $properties = $config->get('properties') ?? [];
    $triggering_element = $form_state->getTriggeringElement();
    $button_name = $triggering_element['#name'];

    if (str_starts_with($button_name, 'delete')) {
      [$action, $id] = explode('__', $button_name);
      unset($properties[$id]);
    }
    else {
      foreach ($form_state->getValues() as $key => $value) {
        if (str_ends_with($key, '_container') && !empty($value['id'])) {
          $properties[$value['id']] = [
            'label' => $value['label'],
            'color' => $value['color'],
          ];
        }
      }
    }

    $config
      ->set('custom_property_prefix', $prefix)
      ->set('properties', $properties)
      ->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * Get config object for Custom Properties.
   *
   * @return \Drupal\Core\Config\Config
   *   A config object.
   */
  protected function getConfig(): Config {
    return $this->config('css_custom_properties.settings');
  }

  /**
   * Create a Color setting form widget.
   *
   * @param string $id
   *   The color ID.
   * @param string $label
   *   The label of the color.
   * @param string $color
   *   The hex value of the color.
   *
   * @return array
   *   The form elements for the widget.
   */
  protected function getColorWidget(
    string $id = '',
    string $label = '',
    string $color = '',
  ): array {
    $form_id = !empty($id) ? $id : 'new';
    $form = [];

    $form["{$form_id}_container"] = [
      '#type' => 'fieldset',
      '#title' => $this->t('@label Settings', [
        '@label' => !empty($label) ? $label : 'New Property',
      ]),
      '#tree' => TRUE,
    ];

    $form["{$form_id}_container"]['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Property Name'),
      '#description' => $this->t('This determines the name of the CSS property.'),
      '#default_value' => $label,
      '#tree' => TRUE,
    ];

    $form["{$form_id}_container"]['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $id,
      '#disabled' => $form_id !== 'new',
      '#required' => FALSE,
      '#tree' => TRUE,
      '#machine_name' => [
        'exists' => [$this, 'colorExists'],
        'source' => [
          $form_id !== 'new'
            ? 'custom_property_list'
            : 'custom_property_add_new',
          "{$form_id}_container",
          'label',
        ],
      ],
    ];

    $form["{$form_id}_container"]['color'] = [
      '#type' => 'color',
      '#title' => $this->t('Color'),
      '#default_value' => $color,
      '#tree' => TRUE,
    ];

    if ($form_id !== 'new') {
      $form["{$form_id}_container"]['actions'] = [
        '#type' => 'actions',
      ];

      $form["{$form_id}_container"]['actions']['delete'] = [
        '#type' => 'submit',
        '#value' => $this->t('Delete'),
        '#name' => "delete__{$form_id}",
        '#button_type' => 'danger',
      ];
    }

    return $form;
  }

  /**
   * Checks if a property ID is already set.
   *
   * @param string $property_id
   *   The property ID to validate.
   *
   * @return bool
   *   Boolean based on validation.
   */
  public function colorExists(string $property_id): bool {
    $config = $this->getConfig();
    $property_ids = array_keys($config->get('properties') ?? []);

    return in_array($property_id, $property_ids);
  }

}
